@@ -53,7 +53,7 @@
 #define MIN_SUPPORTED_CONSENSUS_METHOD 28
 
 /** The highest consensus method that we currently support. */
-#define MAX_SUPPORTED_CONSENSUS_METHOD 29
+#define MAX_SUPPORTED_CONSENSUS_METHOD 30
 
 /**
  * Lowest consensus method where microdescriptor lines are put in canonical
@@ -61,6 +61,10 @@
  **/
 #define MIN_METHOD_FOR_CANONICAL_FAMILIES_IN_MICRODESCS 29
 
+/** Lowest consensus method where an unpadded base64 onion-key-ntor is allowed
+ * See #7869 */
+#define MIN_METHOD_FOR_UNPADDED_NTOR_KEY 30
+
 /** Default bandwidth to clip unmeasured bandwidths to using method >=
  * MIN_METHOD_TO_CLIP_UNMEASURED_BW.  (This is not a consensus method; do not
  * get confused with the above macros.) */
@@ -179,6 +183,8 @@ dirvote_add_signatures(const char *detached_signatures_body,
 /* Item access */
 MOCK_DECL(const char*, dirvote_get_pending_consensus,
           (consensus_flavor_t flav));
+MOCK_DECL(uint32_t,dirserv_get_bandwidth_for_router_kb,
+        (const routerinfo_t *ri));
 MOCK_DECL(const char*, dirvote_get_pending_detached_signatures, (void));
 const cached_dir_t *dirvote_get_vote(const char *fp, int flags);
 
@@ -230,6 +236,22 @@ int networkstatus_add_detached_signatures(networkstatus_t *target,
                                           const char *source,
                                           int severity,
                                           const char **msg_out);
+STATIC int
+compare_routerinfo_usefulness(const routerinfo_t *first,
+                              const routerinfo_t *second);
+STATIC
+int compare_routerinfo_by_ipv4(const void **a, const void **b);
+
+STATIC
+int compare_routerinfo_by_ipv6(const void **a, const void **b);
+
+STATIC
+digestmap_t * get_sybil_list_by_ip_version(
+    const smartlist_t *routers, sa_family_t family);
+
+STATIC
+digestmap_t * get_all_possible_sybil(const smartlist_t *routers);
+
 STATIC
 char *networkstatus_get_detached_signatures(smartlist_t *consensuses);
 STATIC microdesc_t *dirvote_create_microdescriptor(const routerinfo_t *ri,
@@ -238,32 +260,60 @@ STATIC microdesc_t *dirvote_create_microdescriptor(const routerinfo_t *ri,
 /** The recommended relay protocols for this authority's votes.
  * Recommending a new protocol causes old tor versions to log a warning.
  */
-#define DIRVOTE_RECCOMEND_RELAY_PROTO \
-  "Cons=1-2 Desc=1-2 DirCache=1 HSDir=1 HSIntro=3 HSRend=1 " \
-  "Link=4 Microdesc=1-2 Relay=2"
+#define DIRVOTE_RECOMMEND_RELAY_PROTO           \
+  "Cons=2 "                                     \
+  "Desc=2 "                                     \
+  "DirCache=2 "                                 \
+  "HSDir=2 "                                    \
+  "HSIntro=4 "                                  \
+  "HSRend=2 "                                   \
+  "Link=4-5 "                                   \
+  "LinkAuth=3 "                                 \
+  "Microdesc=2 "                                \
+  "Relay=2"
+
 /** The recommended client protocols for this authority's votes.
  * Recommending a new protocol causes old tor versions to log a warning.
  */
-#define DIRVOTE_RECCOMEND_CLIENT_PROTO \
-  "Cons=1-2 Desc=1-2 DirCache=1 HSDir=1 HSIntro=3 HSRend=1 " \
-  "Link=4 Microdesc=1-2 Relay=2"
+#define DIRVOTE_RECOMMEND_CLIENT_PROTO          \
+  "Cons=2 "                                     \
+  "Desc=2 "                                     \
+  "DirCache=2 "                                 \
+  "HSDir=2 "                                    \
+  "HSIntro=4 "                                  \
+  "HSRend=2 "                                   \
+  "Link=4-5 "                                   \
+  "Microdesc=2 "                                \
+  "Relay=2"
 
 /** The required relay protocols for this authority's votes.
  * WARNING: Requiring a new protocol causes old tor versions to shut down.
  *          Requiring the wrong protocols can break the tor network.
  * See Proposal 303: When and how to remove support for protocol versions.
  */
-#define DIRVOTE_REQUIRE_RELAY_PROTO \
-  "Cons=1 Desc=1 DirCache=1 HSDir=1 HSIntro=3 HSRend=1 " \
-  "Link=3-4 Microdesc=1 Relay=1-2"
+#define DIRVOTE_REQUIRE_RELAY_PROTO             \
+  "Cons=2 "                                     \
+  "Desc=2 "                                     \
+  "DirCache=2 "                                 \
+  "HSDir=2 "                                    \
+  "HSIntro=4 "                                  \
+  "HSRend=2 "                                   \
+  "Link=4-5 "                                   \
+  "LinkAuth=3 "                                 \
+  "Microdesc=2 "                                \
+  "Relay=2"
+
 /** The required relay protocols for this authority's votes.
  * WARNING: Requiring a new protocol causes old tor versions to shut down.
  *          Requiring the wrong protocols can break the tor network.
  * See Proposal 303: When and how to remove support for protocol versions.
  */
-#define DIRVOTE_REQUIRE_CLIENT_PROTO \
-  "Cons=1-2 Desc=1-2 DirCache=1 HSDir=1 HSIntro=3 HSRend=1 " \
-  "Link=4 Microdesc=1-2 Relay=2"
+#define DIRVOTE_REQUIRE_CLIENT_PROTO            \
+  "Cons=2 "                                     \
+  "Desc=2 "                                     \
+  "Link=4 "                                     \
+  "Microdesc=2 "                                \
+  "Relay=2"
 
 #endif /* defined(DIRVOTE_PRIVATE) */
 
